Imports System
Imports System.Collections.Generic
Imports System.Text
Imports Office = Microsoft.Office.Core
Imports System.Diagnostics

Public Class CommandBarsMaker

    ' <summary>
    ' Create a new CommandBar (menu/toolbar)
    ' </summary>
    ' <param name="Bars"></param>
    ' <param name="barName"></param>
    ' <param name="Show"></param>
    ' <param name="Enabled"></param>
    ' <returns></returns>
    Public Function NewBar(ByVal Bars As Office.CommandBars, ByVal barName As String, _
    ByVal Show As Boolean, ByVal Enabled As Boolean) As Office.CommandBar
        Dim _bar As Office.CommandBar = Nothing

        Try
            Dim j As Integer = Bars.Count
            Dim i As Integer

            For i = j To 1 Step -1
                _bar = Bars.Item(i)
                If (_bar.Name = barName) Then
                    _bar.Delete()
                End If
            Next

            _bar = TryCast(Bars.Add(barName, , , True), Office.CommandBar)

            If _bar IsNot Nothing Then
                _bar.Enabled = Enabled
                _bar.Visible = Show
            End If
        Catch
            _bar = Nothing
        End Try

        Return _bar
    End Function

    ' <summary>
    ' Create a new CommandBarPopup (pop-up menu)
    ' </summary>
    ' <param name="Bar"></param>
    ' <param name="popName"></param>
    ' <param name="Tag"></param>
    ' <param name="Show"></param>
    ' <param name="ProgID"></param>
    ' <param name="ShowDivider"></param>
    ' <param name="TipText"></param>
    ' <param name="Enabled"></param>
    ' <returns></returns>
    Public Function NewMenu(ByVal Bar As Office.CommandBar, ByVal popName As String, _
    ByVal Tag As Object, ByVal Show As Boolean, ByVal ProgID As String, _
    ByVal ShowDivider As Boolean, ByVal TipText As String, ByVal Enabled As Boolean) As Office.CommandBarPopup
        Dim _pop As Office.CommandBarPopup = Nothing

        Try
            _pop = TryCast(Bar.FindControl(Office.MsoControlType.msoControlPopup, , Tag, , True), Office.CommandBarPopup)

            If (_pop IsNot Nothing) Then
                _pop.Delete(False)
                _pop = Nothing
            End If

            _pop = TryCast(Bar.Controls.Add(Office.MsoControlType.msoControlPopup, , , , True), Office.CommandBarPopup)

            If _pop IsNot Nothing Then
                With _pop
                    .Tag = Tag.ToString()
                    .Caption = popName
                    .Visible = Show
                    .Enabled = Enabled
                    .BeginGroup = ShowDivider
                    .OnAction = "<!" & ProgID & ">"
                    .TooltipText = TipText
                End With
            End If
        Catch
            _pop = Nothing
        End Try

        Return _pop
    End Function

    ' <summary>
    ' Create a new Button and add it to the specified CommandBar (menu/toolbar)
    ' </summary>
    ' <param name="Bar"></param>
    ' <param name="buttonName"></param>
    ' <param name="Tag"></param>
    ' <param name="Show"></param>
    ' <param name="ProgID"></param>
    ' <param name="Style"></param>
    ' <param name="ShowDivider"></param>
    ' <param name="TipText"></param>
    ' <param name="Enabled"></param>
    ' <returns></returns>
    Public Function NewButton(ByVal Bar As Office.CommandBar, _
    ByVal buttonName As String, ByVal Tag As String, ByVal Show As Boolean, ByVal ProgID As String, _
    ByVal Style As Office.MsoButtonStyle, ByVal ShowDivider As Boolean, _
    ByVal TipText As String, ByVal Enabled As Boolean) As Office.CommandBarButton

        Dim _button As Office.CommandBarButton = Nothing

        Try
            _button = TryCast(Bar.FindControl(, , Tag, , True), Office.CommandBarButton)

            If (_button IsNot Nothing) Then
                _button.Delete(False)
            End If
        Catch ex As Exception
            Debug.WriteLine(ex.Message)
        End Try

        Dim buttonType As Object = Office.MsoControlType.msoControlButton

        _button = TryCast(Bar.Controls.Add(buttonType, , , , True), Office.CommandBarButton)

        If (_button IsNot Nothing) Then
            With _button
                .Caption = buttonName
                .Visible = Show
                .Enabled = Enabled
                .Style = Style
                .BeginGroup = ShowDivider
                .Tag = Tag
                .OnAction = "<!" & ProgID & ">"
                .TooltipText = TipText
            End With
        End If
        Return _button
    End Function

    ' <summary>
    ' Create a new ComboBox and add it to the specified CommandBar (menu/toolbar)
    ' </summary>
    ' <param name="Bar"></param>
    ' <param name="comboName"></param>
    ' <param name="Tag"></param>
    ' <param name="Show"></param>
    ' <param name="ProgID"></param>
    ' <param name="Style"></param>
    ' <param name="ShowDivider"></param>
    ' <param name="TipText"></param>
    ' <param name="Enabled"></param>
    ' <returns></returns>
    Public Function NewComboBox(ByVal Bar As Office.CommandBar, _
        ByVal comboName As String, ByVal Tag As String, ByVal Show As Boolean, _
        ByVal ProgID As String, ByVal Style As Office.MsoComboStyle, _
        ByVal ShowDivider As Boolean, ByVal TipText As String, _
        ByVal Enabled As Boolean) As Office.CommandBarComboBox

        Dim _combo As Office.CommandBarComboBox = Nothing

        Try
            _combo = TryCast(Bar.FindControl(, , Tag, , True), Office.CommandBarComboBox)

            If (_combo IsNot Nothing) Then
                _combo.Delete(False)
            End If
        Catch ex As Exception
            Debug.WriteLine(ex.Message)
        End Try

        Dim controlType As Object = Office.MsoControlType.msoControlComboBox

        _combo = TryCast(Bar.Controls.Add(controlType, , , , True), Office.CommandBarComboBox)

        If (_combo IsNot Nothing) Then
            With _combo
                .Caption = comboName
                .Visible = Show
                .Enabled = Enabled
                .Style = Style
                .BeginGroup = ShowDivider
                .Tag = Tag
                .OnAction = "<!" & ProgID & ">"
                .TooltipText = TipText
            End With
        End If

        Return _combo
    End Function
End Class
