Imports Office = Microsoft.Office.Core
Imports Outlook = Microsoft.Office.Interop.Outlook
Imports System.Windows.Forms
Imports System.Runtime.InteropServices
Imports System.Reflection
Imports Microsoft.Office.Tools

' When using this template make sure to change the AssemblyInfo module to use your own 
'   settings and change the addin GUID to a new unique GUID, as well as changing the 
'   ProgId for the addin.
'
' Also change the target folder for deployment in the Setup project.
'
'    
' This template doesn't include the SetSecurity project needed for setting CAS
'   security so the VSTO addin can load in Outlook. Download the SetSecurity 
'   project and the VSTO 2005 SE deployment walkthrough from the MSDN Web
'   site and use that material to successfully deploy a VSTO addin.

Partial Public Class ThisAddIn

#Region "Class level declarations"
    Private m_objOutlook As Outlook.Application

    Private WithEvents m_objNS As Outlook.NameSpace

    'Event-aware references to Explorers collection & ActiveExplorer
    Private WithEvents m_colExplorers As Outlook.Explorers
    Private WithEvents m_olExplorer As Outlook.Explorer

    'Event-aware references to Inspectors collection & ActiveInspector
    Private WithEvents m_colInspectors As Outlook.Inspectors
    Private WithEvents m_olInspector As Outlook.Inspector

    ' State flags
    Private m_blnTeardown As Boolean
    Private m_blnInit As Boolean
    Private _disposed As Boolean

    Private m_strProgID As String

#End Region

#Region "Startup and Shutdown"

    Private Sub ThisAddIn_Startup(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Startup
        ' Store a reference to the Outlook.Application object.
        m_objOutlook = Me.Application

        Try
            AddinObject = New AutomationObject(Me)
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try

        g_ProgID = ThisAddIn.ProgId

        m_blnTeardown = False

        Try
            'event-aware reference to Explorers collection
            'use NewExplorer event to watch for UI creation
            m_colExplorers = m_objOutlook.Explorers

            Try
                'Are we starting with UI?
                If m_colExplorers.Count > 0 Then
                    m_olExplorer = m_objOutlook.Explorers.Item(1)

                    'we have UI 
                    InitHandler()

                    If m_blnInit = True Then
                        AddExpl(m_olExplorer)
                    End If

                    m_objNS = m_objOutlook.GetNamespace("MAPI")

                    m_colInspectors = m_objOutlook.Inspectors
                Else
                    'do nothing
                    'monitor Explorers collection (in this module)
                    'if NewExplorer event is raised then we have UI
                End If
            Catch ex As Exception
                TearDown()
            End Try

        Catch ex As Exception
            TearDown()
        End Try

    End Sub

    Private Sub ThisAddIn_Shutdown(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Shutdown
        ' clean up the Outlook objects and other public objects.
        TearDown()
    End Sub
#End Region

    Public Shared ReadOnly Property ProgId() As String
        Get
            Dim thisAssemblyName As AssemblyName = Assembly.GetExecutingAssembly.GetName()
            Return thisAssemblyName.Name
        End Get
    End Property

#Region "Utility procedures"
    Private Sub InitHandler()
        Dim sVerLeft2 As String

        Try
            g_objOL = m_objOutlook
            g_objNS = g_objOL.GetNamespace("MAPI")

            '********************************************************

            sVerLeft2 = Left(g_objOL.Version, 2)
            Select Case sVerLeft2
                Case "10"
                    g_OutlookVersion = 10
                Case "11"
                    g_OutlookVersion = 11
                Case "12"
                    g_OutlookVersion = 12
                Case Else
                    If Left(sVerLeft2, 1) = "9" Then
                        g_OutlookVersion = 9
                    Else
                        g_OutlookVersion = 0
                    End If
            End Select

            '********************************************************

            m_blnInit = True
        Catch ex As Exception
            MessageBox.Show(ex.Message)

            If m_blnTeardown = False Then
                TearDown()
            End If
        End Try
    End Sub

    Private Sub TearDown()
        Dim i As Integer
        Dim j As Integer

        If m_blnTeardown = False Then
            Try
                If g_colExplWrap IsNot Nothing Then
                    j = g_colExplWrap.Count
                    If j > 0 Then
                        For i = j To 1 Step -1
                            g_colExplWrap.Remove(i)
                        Next
                    End If
                    g_colExplWrap = Nothing
                End If

                If g_colInspWrap IsNot Nothing Then
                    j = g_colInspWrap.Count
                    If j > 0 Then
                        For i = j To 1 Step -1
                            g_colInspWrap.Remove(i)
                        Next
                    End If
                    g_colInspWrap = Nothing
                End If

            Catch ex As Exception
                MessageBox.Show(ex.Message)
            End Try

            If g_objNS IsNot Nothing Then
                g_objNS = Nothing
            End If

            Try
                'release reference to Outlook objects
                m_olExplorer = Nothing
                m_colExplorers = Nothing
                m_colInspectors = Nothing
                m_olInspector = Nothing
                m_objNS = Nothing
                m_objOutlook = Nothing
            Catch ex As Exception
                MessageBox.Show(ex.Message)
            End Try

            If g_objOL IsNot Nothing Then
                g_objOL = Nothing
            End If

            m_blnTeardown = True
            m_blnInit = False
        End If
    End Sub

#End Region

#Region "Explorer (folder window) related events"
    ' NewExplorer event will be raised if there is UI
    Private Sub m_colExplorers_NewExplorer(ByVal Explorer As Microsoft.Office.Interop.Outlook.Explorer) Handles m_colExplorers.NewExplorer
        'assign ActiveExplorer
        m_olExplorer = Explorer

        If m_blnInit = False Then
            'we didn't have UI before - initialize add-in objects
            InitHandler()
        End If

        If m_blnInit = True Then
            AddExpl(Explorer)
        End If
    End Sub

    ' Monitor Explorer_Close to see when UI "disappears"
    Private Sub m_olExplorer_Close() Handles m_olExplorer.Close
        'release current reference
        m_olExplorer = Nothing

        Try
            m_olExplorer = m_objOutlook.ActiveExplorer
        Catch ex As Exception
            'release add-in objects
            If m_colInspectors.Count = 0 Then
                If m_blnTeardown = False Then
                    TearDown()
                End If
            End If
        End Try
    End Sub
#End Region

#Region "Inspector (item window) related events"
    Private Sub m_colInspectors_NewInspector(ByVal Inspector As Microsoft.Office.Interop.Outlook.Inspector) Handles m_colInspectors.NewInspector
        'No handling of Inspectors for Notes, they are brain dead
        Dim CTP As Office.CustomTaskPane

        If Not (TypeOf (Inspector.CurrentItem) Is Outlook.NoteItem) Then
            m_olInspector = Inspector

            If TypeOf (Inspector.CurrentItem) Is Outlook.MailItem Then
                ' You can move this code to the Inspector wrapper class,
                '  that way you can have individualized task panes.
                ' To do so make sure the TaskPaneFactory object is available.
                '
                ' change the strings for your own addin
                CTP = TaskPaneWrapper.m_CTP.CreateCTP(CTP_PROGID, "Configure Settings", Inspector)

                CTP.DockPositionRestrict = Office.MsoCTPDockPositionRestrict.msoCTPDockPositionRestrictNoChange
                CTP.DockPosition = Office.MsoCTPDockPosition.msoCTPDockPositionRight
                CTP.Visible = False
            Else
                CTP = Nothing
            End If

            'wrap and ribbon are dummies, filled in when Ribbon_OnLoad and BeforeFormRegionShow fire
            Dim wrap As New TimeTravelFormRegionWrapper()
            AddInsp(Inspector, CTP, wrap, ribbon)
            wrap = Nothing
        End If

        CTP = Nothing
    End Sub

    Private Sub m_olInspector_Close() Handles m_olInspector.Close
        m_olInspector = Nothing

        Try
            m_olInspector = m_objOutlook.ActiveInspector
        Catch ex As Exception
            If m_objOutlook.Explorers.Count = 0 Then
                If m_blnTeardown = False Then
                    TearDown()
                End If
            End If
        End Try
    End Sub
#End Region

#Region "Property Page display event"
    ' NameSpace Property Pages Add event is handled here. 
    Private Sub m_objNS_OptionsPagesAdd(ByVal Pages As Microsoft.Office.Interop.Outlook.PropertyPages, ByVal Folder As Microsoft.Office.Interop.Outlook.MAPIFolder) Handles m_objNS.OptionsPagesAdd
        Dim Page As New FolderPP()
        Dim caption As String

        ' change the strings for your own addin
        caption = "VSTO VB Folder " & Folder.Name & " Property Page"
        Page.Text = caption

        Pages.Add(Page)

        Page = Nothing
    End Sub

    ' Application events handled by Application object.
    Private Sub Application_OptionsPagesAdd(ByVal Pages As Microsoft.Office.Interop.Outlook.PropertyPages) Handles Application.OptionsPagesAdd
        Dim Page As New ToolsOptionsPP()
        Dim caption As String

        ' change the strings for your own addin
        caption = "VSTO VB Options Property Page"
        Page.Text = caption

        Pages.Add(Page)

        Page = Nothing
    End Sub
#End Region

    Private myUtilities As AddinUtilities

    Protected Overrides Function RequestCOMAddInAutomationService() As Object
        If (myUtilities Is Nothing) Then
            myUtilities = New AddinUtilities()
        End If

        Return myUtilities
    End Function

End Class

<ComVisibleAttribute(True), InterfaceType(ComInterfaceType.InterfaceIsDual)> _
Public Interface IAddinUtilities
    Sub CalledFromOutside()
End Interface

<ComVisibleAttribute(True), ClassInterface(ClassInterfaceType.None)> _
Public Class AddinUtilities
    Implements IAddinUtilities

    'Demonstrates a method that can be called from outside the addin.
    ' This technique can be used to call functions and to read/write properties.

    Sub CalledFromOutside() Implements IAddinUtilities.CalledFromOutside
        Dim myAddinObject As AutomationObject = AddinObject
        Dim Count As String = myAddinObject.ExplCount

        'MessageBox.Show("This is a test of an outside call to the COM addin")
        MessageBox.Show("There are currently " & Count & " Explorers open.")
    End Sub
End Class
